
//`#start header` -- edit after this line, do not edit this line
// ===============================================================
// SeqDetect_v1_00.v
// 
// Parameter: 
// uint8 sequence - decimal equivalent of sequence to be detected 
//
// Inputs:
// zeroIn	- Active high indication of input value of zero
// oneIn	- Active high indication of input value of one
// reset	- Synchronous reset
// clock	- Operating frequency of component
//
// Outputs:
// detect 	- Logic high iff complete sequence is detected
// restart 	- Logic high iff a partially wrong input is entered
//
// The 5-bit sequence detector takes in a sequence of inputs and 
// sets its 'Detect' output to 1 if the correct input sequence is
// detected. The component stays in the 'Detect' state until it
// gets another input. 
// The 'restart' output indicates whether the user has 
// has entered a partially correct sequence or not. Initially, 
// restart = 1. If the user then provides a partially correct 
// sequence, restart = 0. Entering a wrong value causes 
// restart to go back to 1. 
// Note that the sequence detector does not detect overlapping 
// sequences.
// ===============================================================

`include "cypress.v"
//`#end` -- edit above this line, do not edit this line
// Generated on 08/30/2012 at 20:17
// Component: SeqDetect_v1_00
module SeqDetect_v1_00 (
	output reg detect,
	output reg restart,
	input   clock,
	input   oneIn,
	input   reset,
	input   zeroIn
);
	/* Note that the value assigned to the parameter in this line
	 * has no effect. The actual parameter value is taken from 
	 * the component customizer.
	 */
	parameter sequence = 0;

//`#start body` -- edit after this line, do not edit this line
	/* 5-bit sequence implies 6 states are required
	 * The states follow START -> STATE_1 -> ... -> DETECT if the 
	 * correct inputs are entered. As soon as a wrong input is entered
	 * the design jumps to the START state. The states are defined as 
	 * localparams (instead of `defines to limit their scope to this 
	 * module only. Note that the state bits are gray coded. 
	 */ 
	localparam START   = 3'b000;		/* detect, restart = 0, 1 */
	localparam STATE_1 = 3'b001;		/* detect, restart = 0, 0 */
	localparam STATE_2 = 3'b011;		/* detect, restart = 0, 0 */
	localparam STATE_3 = 3'b010;		/* detect, restart = 0, 0 */
	localparam STATE_4 = 3'b110;		/* detect, restart = 0, 0 */
	localparam DETECT  = 3'b100;		/* detect, restart = 1, 0 */
	
	/* registered value to hold 3-bit state */
	reg [2:0] state_curr, state_next;
	
	/* pattern[4:0] holds the user-supplied sequence value 
	 * suppose sequence = 22 then pattern[4:0] = 5'b10110
	 * Note that pattern[4] is the first-entered user input
	 */
	wire [4:0] pattern = sequence;

	 /* Sequential block of the state machine - outputs are assigned here */
	always @ (posedge clock)
	begin
		/* reset causes the component to enter the START state */
		if(reset)
		begin	
			state_curr <= START;
			/* Immediately assign detect and restart values */
			detect <= 1'b0;
			restart <= 1'b1;
		end
		else	/* reset is not asserted - go through states */
		begin
			state_curr <= state_next;
			
			/* Assign 'detect' value - 1 only in DETECT state, 0 otherwise */
			if (state_next == DETECT) 
			begin
				detect <= 1'b1;
			end
			else
			begin
				detect <= 1'b0;
			end
			
			/* Assign 'restart' value - 1 only in RESTART state, 0 otherwise */
			if (state_next == START) 
			begin
				restart <= 1'b1;
			end
			else
			begin
				restart <= 1'b0;
			end
		end
	end
	
	/* Finite State Machine combinatorial block - contains most of the 
	 * combinational logic.
	 */
	always @ (oneIn or zeroIn or state_curr or pattern) 
	begin
		case(state_curr)
			START:		/* Initial state */
			begin
				/* If either a one or zero has been entered, take action */
				if(oneIn | zeroIn)
				begin
					/* check whether the first bit entered is correct */
					if((oneIn & pattern[4]) || (zeroIn & !pattern[4]))
					begin
						state_next <= STATE_1; 	/* advance to the next state */
					end
					else 						/* revert to the initial state */
					begin	
						state_next <= START;
					end
				end
				else 	/* if neither 1 or 0 have been entered, stay in same state */
				begin
					state_next <= state_curr;
				end
			end
			
			STATE_1:	/* First input is correct */
			begin
				if(oneIn | zeroIn)
				begin
					if((oneIn & pattern[3]) || (zeroIn & !pattern[3]))
					begin
						state_next <= STATE_2;
					end
					else
					begin
						state_next <= START;
					end
				end
				else
				begin
					state_next <= state_curr;
				end
			end				
			
			STATE_2:	/* Two inputs are correct */
			begin
				if(oneIn | zeroIn)
				begin
					if((oneIn & pattern[2]) || (zeroIn & !pattern[2]))
					begin	
						state_next <= STATE_3;
					end
					else
					begin
						state_next <= START;
					end
				end
				else
				begin	
					state_next <= state_curr;
				end
			end				
			
			STATE_3:	/* Three inputs are correct */
			begin
				if(oneIn | zeroIn)
				begin
					if((oneIn & pattern[1]) || (zeroIn & !pattern[1]))
					begin	
						state_next <= STATE_4;
					end
					else
					begin
						state_next <= START;
					end
				end
				else
				begin
					state_next <= state_curr;
				end
			end		
			
			STATE_4:	 /* Four inputs are correct */
			begin
				if(oneIn | zeroIn)
				begin
					if((oneIn & pattern[0]) || (zeroIn & !pattern[0]))
					begin
						state_next <= DETECT;
					end
					else
					begin
						state_next <= START;
					end
				end
				else
				begin
					state_next <= state_curr;
				end
			end
			
			DETECT:		/* All five inputs are correct! */
			begin
				/* When in the detect state, if an input is given, show same behavior as START */
				if((oneIn | zeroIn))
				begin
					/* check whether the bit entered is the correct beginnning to a new sequence*/
					if((oneIn & pattern[4]) || (zeroIn & !pattern[4]))
					begin	
						state_next <= STATE_1;
					end
					else 	/* revert to the initial state */
					begin	
						state_next <= START;
					end
				end
				else
				begin
					state_next <= state_curr;
				end
			end
			
			default:	 /* we should never get here - reset the component */
			begin
				state_next <= START;
			end
		endcase
	end
	
//`#end` -- edit above this line, do not edit this line
endmodule
//`#start footer` -- edit after this line, do not edit this line
//`#end` -- edit above this line, do not edit this line
